==> async_wrapper-0.1.1.tar.gz: async_wrapper-0.1.1/src/async_wrapper/task_group/base.py <==
from __future__ import annotations

from abc import ABC, abstractmethod
from threading import local
from typing import Any, Awaitable, Callable, Generic, TypeVar

from typing_extensions import ParamSpec, override

from async_wrapper.convert.synclib.base import as_coro_func

TaskGroupT = TypeVar("TaskGroupT")
ValueT = TypeVar("ValueT")
ValueT_co = TypeVar("ValueT_co", covariant=True)
OtherValueT_co = TypeVar("OtherValueT_co", covariant=True)
ParamT = ParamSpec("ParamT")
OtherParamT = ParamSpec("OtherParamT")
Pending = local()

__all__ = ["PendingError", "BaseSoonWrapper", "SoonValue"]


class PendingError(Exception):
    ...


class BaseSoonWrapper(ABC, Generic[TaskGroupT, ParamT, ValueT_co]):
    def __init__(
        self,
        func: Callable[ParamT, Awaitable[ValueT_co]],
        task_group: TaskGroupT,
    ) -> None:
        self.func = as_coro_func(func)
        self.task_group = task_group

    @override
    def __new__(
        cls,
        func: Callable[OtherParamT, Awaitable[OtherValueT_co]],
        task_group: TaskGroupT,
    ) -> BaseSoonWrapper[TaskGroupT, OtherParamT, OtherValueT_co]:
        return super().__new__(cls)  # type: ignore

    @abstractmethod
    def __call__(  # noqa: D102
        self,
        *args: ParamT.args,
        **kwargs: ParamT.kwargs,
    ) -> SoonValue[ValueT_co]:
        ...


class SoonValue(Generic[ValueT_co]):
    def __init__(self) -> None:
        self._value = Pending

    @property
    def value(self) -> ValueT_co:  # noqa: D102
        if self._value is Pending:
            raise PendingError
        return self._value  # type: ignore

    @value.setter
    def value(self, value: Any) -> None:
        self._value = value

    @value.deleter
    def value(self) -> None:
        raise NotImplementedError

    @property
    def is_ready(self) -> bool:  # noqa: D102
        return self._value is not Pending

==> async_wrapper-0.1.3.tar.gz: async_wrapper-0.1.3/src/async_wrapper/task_group/base.py <==
from __future__ import annotations

from abc import ABC, abstractmethod
from threading import local
from typing import (
    Any,
    Awaitable,
    Callable,
    Generic,
    Protocol,
    TypeVar,
)

from typing_extensions import ParamSpec, override

from async_wrapper.convert.synclib.base import as_coro_func

TaskGroupT = TypeVar("TaskGroupT")
TaskGroupT_co = TypeVar("TaskGroupT_co", covariant=True)
ValueT = TypeVar("ValueT")
ValueT_co = TypeVar("ValueT_co", covariant=True)
OtherValueT_co = TypeVar("OtherValueT_co", covariant=True)
ParamT = ParamSpec("ParamT")
OtherParamT = ParamSpec("OtherParamT")
Pending = local()

__all__ = ["PendingError", "BaseSoonWrapper", "SoonValue", "TaskGroupFactory"]


class PendingError(Exception):
    ...


class BaseSoonWrapper(ABC, Generic[TaskGroupT, ParamT, ValueT_co]):
    def __init__(
        self,
        func: Callable[ParamT, Awaitable[ValueT_co]],
        task_group: TaskGroupT,
    ) -> None:
        self.func = as_coro_func(func)
        self.task_group = task_group

    @override
    def __new__(
        cls,
        func: Callable[OtherParamT, Awaitable[OtherValueT_co]],
        task_group: TaskGroupT,
    ) -> BaseSoonWrapper[TaskGroupT, OtherParamT, OtherValueT_co]:
        return super().__new__(cls)  # type: ignore

    @abstractmethod
    def __call__(  # noqa: D102
        self,
        *args: ParamT.args,
        **kwargs: ParamT.kwargs,
    ) -> SoonValue[ValueT_co]:
        ...


class SoonValue(Generic[ValueT_co]):
    def __init__(self) -> None:
        self._value = Pending

    @property
    def value(self) -> ValueT_co:  # noqa: D102
        if self._value is Pending:
            raise PendingError
        return self._value  # type: ignore

    @value.setter
    def value(self, value: Any) -> None:
        self._value = value

    @value.deleter
    def value(self) -> None:
        raise NotImplementedError

    @property
    def is_ready(self) -> bool:  # noqa: D102
        return self._value is not Pending


class TaskGroupFactory(Protocol[TaskGroupT_co]):
    def __call__(self) -> TaskGroupT_co:  # noqa: D102
        ...
